//---------------------------------------------------------------------------
const P11CAPI_EXTENSION_NAME = "capi-bridge@keyon.ch";
const P11CAPI_MODULE1_NAME = "keyon MS-CAPI Bridge (MY)";
const P11CAPI_MODULE1_PATH_X86 = "components/WINNT_x86-msvc/p11capi.dll";
const P11CAPI_MODULE1_PATH_X64 = "components/WINNT_x86_64-msvc/p11capi.dll";
const P11CAPI_MODULE2_NAME = "keyon MS-CAPI Bridge (ROOT)";
const P11CAPI_MODULE2_PATH_X86 = "components/WINNT_x86-msvc/roots.dll";
const P11CAPI_MODULE2_PATH_X64 = "components/WINNT_x86_64-msvc/roots.dll";

//---------------------------------------------------------------------------
Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import("resource://gre/modules/AddonManager.jsm");

//---------------------------------------------------------------------------
const Ci = Components.interfaces;
const Cc = Components.classes;

//---------------------------------------------------------------------------
const nsIPKCS11 = Ci.nsIPKCS11;
const nsPKCS11ContractID = "@mozilla.org/security/pkcs11;1";

//---------------------------------------------------------------------------
const PREF_BRANCH = "extensions.capi-bridge.";
const PREFS = {
	debug: false,
};

//---------------------------------------------------------------------------
function setDefaultPrefs() {
	let branch = Services.prefs.getDefaultBranch(PREF_BRANCH);
	for (let [key, val] in Iterator(PREFS)) {
		switch (typeof val) {
		case "boolean":
			branch.setBoolPref(key, val);
			break;
		case "number":
			branch.setIntPref(key, val);
			break;
		case "string":
			branch.setCharPref(key, val);
			break;
		}
	}
}

//---------------------------------------------------------------------------
function getPref(key) {
	try {
		if (getPref.branch == null) {
		getPref.branch = Services.prefs.getBranch(PREF_BRANCH);
		}
		switch (typeof PREFS[key]) {
		case "boolean":
			return getPref.branch.getBoolPref(key);
		case "number":
			return getPref.branch.getIntPref(key);
		case "string":
			return getPref.branch.getCharPref(key);
		}
	}
	catch(ex) {
		return false;
	}
}

//---------------------------------------------------------------------------
function removeUserPrefs() {
	let userBranch = Services.prefs.getBranch(PREF_BRANCH);
	for (let [key, val] in Iterator(PREFS)) {
		try {
			userBranch.clearUserPref(key);
		}
		catch(ex) {
		}
	}
}

//---------------------------------------------------------------------------
// debug support
function debugLog(logMessage) {
	if (getPref("debug")) {
		const console = Cc['@mozilla.org/consoleservice;1'].getService(Ci.nsIConsoleService);
		console.logStringMessage("CAPI-Bridge: "+logMessage);
	}
}

//---------------------------------------------------------------------------
// bootstrap.js API
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
function startup(aData, aReason) {
	debugLog("startup "+aReason);
	
	// check if we have an enable or install reason
	if (aReason==ADDON_ENABLE || aReason==ADDON_INSTALL || aReason==ADDON_UPGRADE || aReason==ADDON_DOWNGRADE) {

		// Firefox >= 4.0, we have the AddonManager available
		AddonManager.getAddonByID(P11CAPI_EXTENSION_NAME, function(addon) {

			// get the PKCS11 service
			var PKCS11 = Components.classes[nsPKCS11ContractID].getService(nsIPKCS11);			

			// get correct DLL depending on architecture
			var module1Path = P11CAPI_MODULE1_PATH_X86;
			var module2Path = P11CAPI_MODULE2_PATH_X86;
			if (Services.appinfo.is64Bit) {
				module1Path = P11CAPI_MODULE1_PATH_X64;
				module2Path = P11CAPI_MODULE2_PATH_X64;
			}
			
			// install the p11capi.dll
            var dllLocation = addon.getResourceURI(module1Path).QueryInterface(Components.interfaces.nsIFileURL).file.path;
			debugLog("Loading PKCS#11 library from "+dllLocation);
			PKCS11.addModule(P11CAPI_MODULE1_NAME, dllLocation, 0, 0);
			
			// install the roots.dll
            dllLocation = addon.getResourceURI(module2Path).QueryInterface(Components.interfaces.nsIFileURL).file.path;
			debugLog("Loading PKCS#11 library from "+dllLocation);
			PKCS11.addModule(P11CAPI_MODULE2_NAME, dllLocation, 0, 0);
        });

	}
}

//---------------------------------------------------------------------------
function shutdown(aData, aReason) {
	debugLog("shutdown "+aReason);

	// check if we have a disable or uninstall reason
	if (aReason==ADDON_DISABLE || aReason==ADDON_UNINSTALL || aReason==ADDON_UPGRADE || aReason==ADDON_DOWNGRADE) {

		// get the PKCS11 service
		var PKCS11 = Components.classes[nsPKCS11ContractID].getService(nsIPKCS11);
		
		// try to unload p11capi.dll
		try {
		debugLog("Unloading PKCS#11 module "+P11CAPI_MODULE1_NAME);
			PKCS11.deleteModule(P11CAPI_MODULE1_NAME);
		}
		catch(e) {
			// ignore
		}

		// try to unload roots.dll
		try {
		debugLog("Unloading PKCS#11 module "+P11CAPI_MODULE2_NAME);
			PKCS11.deleteModule(P11CAPI_MODULE2_NAME);
		}
		catch(e) {
			// ignore
		}
	}
}

//---------------------------------------------------------------------------
function install(aData, aReason) { 
	debugLog("install "+aReason);
	
	// make sure we delete the module in case it is still loaded (fix to support
	// upgrading an older version of the plugin which did not handling upgrade
	// reasons properly)
	shutdown(aData, ADDON_UNINSTALL);
}

//---------------------------------------------------------------------------
function uninstall(aData, aReason) { 
	debugLog("uninstall "+aReason);
}
